package hdm.mi.growbros.controllers;

import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.service.GrandmaService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/api/v1/Grandma")
public class GrandmaController {

    private final GrandmaService grandmaService;

    public GrandmaController(GrandmaService grandmaService) {
        this.grandmaService = grandmaService;
    }

    @GetMapping("/getGrowingTips")
    @Operation(description = "Get growing tip of the day")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved growing tip"),
    })
    public ResponseEntity<String> getGrowingTipDesTages(
            @AuthenticationPrincipal User user
    ) {
        String response = grandmaService.growingTippDesTages(user);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @GetMapping ("/getGrowablePlants")
    @Operation(description = "Get plants ready to grow")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved plants ready to grow"),
    })
    public ResponseEntity<List<Plant>> getPlantsReadyToGrow(@AuthenticationPrincipal User user){
            return ResponseEntity.ok(grandmaService.getPlantsReadyToGrow(user));

    }

    @GetMapping ("/getPlantsNeedingWatering")
    @Operation(description = "Get plants that need to be watered")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved plants that need to be watered"),
    })
    public ResponseEntity<List<GardenEntry>> getPlantsNeedingWatring (@AuthenticationPrincipal User user){
        return ResponseEntity.ok(grandmaService.getGardenEntriesNeedingWatering(user));
    }

    @GetMapping("/getPlantsReadyToHarvest")
    @Operation(description = "Get plants ready to harvest")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved plants ready to harvest"),
    })
    public ResponseEntity<List<GardenEntry>> getPlantsReadyToHarvest(@AuthenticationPrincipal User user){
        List<GardenEntry> plantsReadyToHarvest = grandmaService.getPlantsReadyToHarvest(user);
        return ResponseEntity.ok(plantsReadyToHarvest);
    }
}
