package hdm.mi.growbros.controllers;

import hdm.mi.growbros.exceptions.GrowBrosException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

@ControllerAdvice
public class RestErrorHandler extends ResponseEntityExceptionHandler {
    private final static Logger log = LoggerFactory.getLogger(RestErrorHandler.class);

    @ExceptionHandler(GrowBrosException.class)
    protected ResponseEntity<String> handleGrowBrosException(GrowBrosException e) {
        log.error("Handling exception '{}' : {} (Status code {})", e.getClass(), e.getMessage(), e.getStatusCode());
        log.debug("Stacktrace:", e);
        return ResponseEntity.status(e.getStatusCode()).body(e.getMessage());
    }

    @ExceptionHandler(AccessDeniedException.class)
    protected ResponseEntity<String> handleAccessDeniedException() {
        log.warn("handling access denied exception");
        return ResponseEntity.status(401).body("Access denied: role not sufficient");
    }

    @ExceptionHandler
    protected ResponseEntity<Object> handleInternalServerError(RuntimeException e, WebRequest req) {
        log.error("Handling internal server error: {}", e.getClass());
        log.error("Stacktrace:", e);
        return handleExceptionInternal(e, "Internal server error", new HttpHeaders(), HttpStatus.INTERNAL_SERVER_ERROR, req);
    }
}
